$title maximizing volume of 8 gon within sphere of given radius
option limrow=0,limcol=0,solprint=on;

set i /1*8/;
alias(i,j,k);
* fixing the face structure limits the solution.
* really need to choose the optimal face structure.
$ontext
* hexagon solution
set face(i,j,k) /
	1.3.8
	1.6.3

	1.8.2
	1.2.7

	1.4.6
	1.7.4

	4.5.6
	5.3.6

	5.4.7
	7.2.5

	3.5.8
	8.5.2
/;
$offtext
* optimal solution
set face(i,j,k) /
	7.6.1
	7.4.6

	1.2.7
	1.8.2

	3.8.1
	1.6.3

	4.3.6
	4.5.3

	8.5.2
	3.5.8

	7.5.4
	7.2.5
/;

set coord /x,y,z/;

scalar radius /1.0/;

table start(i,coord) 
	x	y	z
1	1	-1	-1
2	-1	1	-1
3	-1	-1	1
4	1	1	1
5	-1	1	1
6	1	-1	1
7	1	1	-1
8	-1	-1	-1 ;

table hexstart(i,coord) 
	x	y	z
1			-1
2	-1	1	
3	-1	-1	
4	1		
5			1
6	1	-1	
7	1	1	
8	-1		 ;
parameter pi;
pi = 4*arctan(1);
hexstart("7","x") = cos(pi/3); 
hexstart("2","x") = cos(2*pi/3); 
hexstart("3","x") = cos(4*pi/3); 
hexstart("6","x") = cos(5*pi/3); 
hexstart("7","y") = sin(pi/3); 
hexstart("2","y") = sin(2*pi/3); 
hexstart("3","y") = sin(4*pi/3); 
hexstart("6","y") = sin(5*pi/3); 

hexstart(i,coord) = radius*hexstart(i,coord);

variables 
	volume,
	posn(i,coord)	'position of points'
	tetVol(i,j,k)	'volume of tetrahedron';

positive variables tetVol;

equations 
	dtetVol(i,j,k),
	inscribe(i),
	defVol;

dtetVol(i,j,k)$face(i,j,k)..
tetVol(i,j,k) =e=
	 (posn(i,'x')*(posn(j,'y')*posn(k,'z')-posn(k,'y')*posn(j,'z'))
      +  posn(i,'y')*(posn(j,'z')*posn(k,'x')-posn(k,'z')*posn(j,'x'))
      +  posn(i,'z')*(posn(j,'x')*posn(k,'y')-posn(k,'x')*posn(j,'y')))/6;

defVol..
volume =e= sum(face,tetVol(face));

inscribe(i)..
sum(coord,sqr(posn(i,coord))) =l= sqr(radius);

posn.lo(i,coord) = -radius;
posn.up(i,coord) = radius;
posn.l(i,coord) = start(i,coord);
$ontext
posn.l(i,coord) = hexstart(i,coord);
posn.fx(i,"z") = 0;
posn.fx("4",coord) = hexstart("4",coord);
posn.fx("1",coord) = hexstart("1",coord);
posn.fx("5",coord) = hexstart("5",coord);
$offtext

option nlp = conopt2;
model ngon /all/;
* ngon.holdfixed = 1;
solve ngon using nlp maximizing volume

set index /1*12/;
set threed /1*3/;
parameter faceval(index,threed)

scalar numface /0/;
loop((i,j,k)$face(i,j,k),
  numface = numface + 1;
  faceval(index,'1')$(ord(index) eq numface) = ord(i);
  faceval(index,'2')$(ord(index) eq numface) = ord(j);
  faceval(index,'3')$(ord(index) eq numface) = ord(k);
);

$libinclude matout posn.l i coord
$libinclude matout faceval index threed
$libinclude matout radius
